/**
   LED灯自定义编程
 * **/
#include "keyboard.h"
#include "ws2812.h"

/**
   name: user_mode0

   传参：

   返回值：

   功能：
    熄灭灯光
   内容：
*/
void user_mode0()
{
  fill_solid(leds, WS2812_NUMBER, CRGB::Black);
  FastLED.show();
  //  FastLED.clear();
}

/**
   name: user_mode1

   传参：

   返回值：

   功能：
    全部点亮
   内容：
**/
void user_mode1()
{
  uint8_t color_temp = 0;
  get_rgb(&color_temp);
  switch (color_temp)
  {
    case 0: fill_solid(leds, WS2812_NUMBER, CRGB::Red); FastLED.show(); break;
    case 1: fill_solid(leds, WS2812_NUMBER, CRGB::Blue); FastLED.show(); break;
    case 2: fill_solid(leds, WS2812_NUMBER, CRGB::Green); FastLED.show(); break;
    case 3: fill_solid(leds, WS2812_NUMBER, CRGB::Yellow); FastLED.show(); break;
    case 4: fill_solid(leds, WS2812_NUMBER, CRGB::Purple); FastLED.show(); break;
    case 5: fill_solid(leds, WS2812_NUMBER, CRGB::Orange); FastLED.show(); break;
    case 6: fill_solid(leds, WS2812_NUMBER, CRGB::Pink); FastLED.show(); break;
    case 7: fill_solid(leds, WS2812_NUMBER, CRGB::Teal); FastLED.show(); break;
  }
}



/*
   name: user_mode2
   功能：
      彩虹呼吸灯
   注：
**/
void user_mode2()
{
  uint8_t delay_time = 0;

  for (int i = 0; i < 256; i++) {
    for (int j = 0; j < WS2812_NUMBER; j++) {
      leds[j] = CHSV(i, 255, 255);
    }
    FastLED.show();
    get_speed(&delay_time);
    FastLED.delay(delay_time);
    if (RGB_mode_change_flag == 1)
    {
      RGB_mode_change_flag = 0;
      return;
    }
  }
  for (int i = 255; i >= 0; i--) {
    for (int j = 0; j < WS2812_NUMBER; j++) {
      leds[j] = CHSV(i, 255, 255);
    }
    FastLED.show();
    get_speed(&delay_time);
    FastLED.delay(delay_time);
    if (RGB_mode_change_flag == 1)
    {
      RGB_mode_change_flag = 0;
      return;
    }
  }
}



/*
   name: user_mode3
   功能：
    跑马灯效果
   注：
**/
#define GROUP_LENGTH 8  // 组合的长度
void user_mode3()
{
  uint8_t delay_time = 0;
  get_speed(&delay_time);
  // 随机生成组合的颜色
  CRGB groupColor = CRGB(random(0, 256), random(0, 256), random(0, 256));

  // 随机生成每个灯的颜色
  CRGB colors[GROUP_LENGTH];
  int sameColorIndex = random(0, GROUP_LENGTH); // 随机选择一个灯的颜色与组合的颜色相同
  for (int i = 0; i < GROUP_LENGTH; i++) {
    if (i == sameColorIndex) {
      colors[i] = groupColor; // 一个灯的颜色与组合的颜色相同
    } else {
      // 其他灯的颜色随机生成，并且每个颜色的RGB分量加上一个随机值
      colors[i] = CRGB(
                    random(groupColor.r - 50, groupColor.r + 50),
                    random(groupColor.g - 50, groupColor.g + 50),
                    random(groupColor.b - 50, groupColor.b + 50)
                  );
    }
  }

  // 绘制组合
  for (int i = 0; i < WS2812_NUMBER; i++) {
    if (i >= 0 && i < GROUP_LENGTH) {
      leds[i] = colors[i]; // 组合的颜色为随机生成的颜色
    } else {
      leds[i] = CRGB::Black; // 灯的颜色为黑色
    }
  }
  if (RGB_mode_change_flag == 1)
  {
    RGB_mode_change_flag = 0;
    return;
  }
  // 显示LED灯条
  FastLED.show();

  // 等待一段时间
  get_speed(&delay_time);
  FastLED.delay(delay_time);

  // 移动组合
  for (int i = 0; i < WS2812_NUMBER - GROUP_LENGTH + 1; i++) {
    // 将组合向右移动一个灯
    for (int j = 0; j < WS2812_NUMBER; j++) {
      if (j >= i && j < i + GROUP_LENGTH) {
        leds[j] = colors[j - i]; // 组合的颜色为随机生成的颜色
      } else {
        leds[j] = CRGB::Black; // 灯的颜色为黑色
      }
    }
    if (RGB_mode_change_flag == 1)
    {
      RGB_mode_change_flag = 0;
      return;
    }
    // 显示LED灯条
    FastLED.show();

    // 等待一段时间
    get_speed(&delay_time);
    FastLED.delay(delay_time);
  }

  // 清除移动到末尾的灯
  for (int i = WS2812_NUMBER - GROUP_LENGTH; i < WS2812_NUMBER; i++) {
    leds[i] = CRGB::Black; // 灯的颜色为黑色
  }

  // 将组合移动到开头来
  for (int i = 0; i < WS2812_NUMBER; i++) {
    if (i >= WS2812_NUMBER - GROUP_LENGTH) {
      leds[i] = colors[i - WS2812_NUMBER + GROUP_LENGTH]; // 组合的颜色为随机生成的颜色
    } else {
      leds[i] = CRGB::Black; // 灯的颜色为黑色
    }
  }
  if (RGB_mode_change_flag == 1)
  {
    RGB_mode_change_flag = 0;
    return;
  }
  // 显示LED灯条
  FastLED.show();

  // 等待一段时间
  get_speed(&delay_time);
  FastLED.delay(delay_time);
}
/*
   name: user_mode4
   功能：
     流星雨灯效
   注：
**/
void user_mode4()
{
  uint8_t delay_time = 0;
  get_speed(&delay_time);
  for (int i = 0; i < WS2812_NUMBER; i++) {
    leds[i] = CRGB::Black;
  }
  for (int i = 0; i < 10; i++) {
    int pos = random(WS2812_NUMBER);
    int len = random(5, 10);
    CRGB color = CHSV(random(0, 255), 255, 255);
    for (int j = 0; j < len; j++) {
      int brightness = map(j, 0, len - 1, 255, 0);
      leds[(pos + j) % WS2812_NUMBER] = color;
      leds[(pos + j) % WS2812_NUMBER].fadeToBlackBy(brightness);
      FastLED.show();
      get_speed(&delay_time);
      FastLED.delay(delay_time);
      color = color + CRGB(10, 10, 10); // 颜色变化
    }
  }
}

/*
   name: user_mode5
   功能：
     星光效果
   注：
**/
void user_mode5()
{
  uint8_t delay_time = 0;
  get_speed(&delay_time);
  // 清空所有LED
  FastLED.clear();

  // 随机选择一种颜色
  CRGB color = CHSV(random8(), 255, 255);

  // 随机选择一些LED点亮
  for (int i = 0; i < WS2812_NUMBER / 10; i++) {
    leds[random8(WS2812_NUMBER)] = color;
  }

  // 使LED逐渐熄灭
  for (int i = 0; i < 20; i++) {
    FastLED.show();
    fadeToBlackBy(leds, WS2812_NUMBER, 10);
    get_speed(&delay_time);
    FastLED.delay(delay_time);
  }
}

/*
   name: user_mode6

   传参：

   返回值：

   功能：
    竖直彩虹波浪
   内容：

   时间：2020/12/2

   注：
**/
void user_mode6()
{
  uint8_t delay_time = 0;
  get_speed(&delay_time);
  static uint8_t hue = 0;

  // 将每个LED设置为不同的彩虹色
  for (int i = 0; i < WS2812_NUMBER; i++) {
    leds[i] = CHSV(hue + (i * 255 / WS2812_NUMBER), 255, 255);
  }

  FastLED.show();

  // 增加彩虹色的色调
  hue += 5;
  get_speed(&delay_time);
  FastLED.delay(delay_time);
}


//按键点灯
void user_mode7()
{
  uint8_t key_state[KEYBOARD_ROW][KEYBOARD_COLUMN] = {OFF};
  if (xQueuePeek(KEYBOARD_SATE_QUEUE, &key_state, 5) == pdTRUE)
  {
    for (uint8_t i = 0; i < KEYBOARD_ROW; i++)
    {
      for (uint8_t j = 0; j < KEYBOARD_COLUMN; j++)
      {
        //按键处于按下状态
        if (key_state[i][j] == 1)
        {
          CRGB groupColor = CRGB(random(0, 256), random(0, 256), random(0, 256));
          leds[light_num[i][j] - 1] = groupColor;
        }
        else
        {
          leds[light_num[i][j] - 1] = CRGB::Black;
        }
      }
    }
    FastLED.show();
  }
}


//水平波浪
uint8_t hue[KEYBOARD_COLUMN]; // 存储每一列的色调
uint8_t pos[KEYBOARD_COLUMN]; // 存储每一列的位置


void user_mode8()
{
  uint8_t delay_time = 0;
  get_speed(&delay_time);
  if (Horizontal_wave_flag == 1)
  {
    Horizontal_wave_flag = 0;
    for (uint8_t i = 0; i < KEYBOARD_COLUMN; i++) { // 初始化每一列的色调和位置
      hue[i] = i * (255 / KEYBOARD_COLUMN);
      pos[i] = i % KEYBOARD_COLUMN;
    }
  }
  for (uint8_t i = 0; i < KEYBOARD_COLUMN; i++) { // 逐列设置颜色
    uint8_t brightness = 255; // 设置颜色亮度为最大值
    for (uint8_t j = 0; j < KEYBOARD_ROW; j++) { // 逐行设置颜色
      if (light_num[j][i] != ERROR_CODE) { // 如果不是错误码，则设置颜色
        leds[light_num[j][i] - 1] = CHSV(hue[i], 255, brightness); // 设置HSV颜色
        brightness -= 40; // 递减颜色亮度
      }
    }
    hue[i] += 5; // 更新色调
    pos[i] = (pos[i] + 1) % KEYBOARD_ROW; // 更新位置
  }

  FastLED.show(); // 显示LED灯效果
  get_speed(&delay_time);
  FastLED.delay(delay_time); // 延迟25毫秒
}

//扫描效果
void user_mode9()
{
  uint8_t delay_time = 0;
  get_speed(&delay_time);
  //  Serial.print("当前速度: ");
  //  Serial.println(delay_time);
  for (int col = 0; col < KEYBOARD_COLUMN; col++) {
    CRGB color = CHSV(random8(), 255, 255);

    for (int row = 0; row < KEYBOARD_ROW; row++) {
      if (light_num[row][col] != ERROR_CODE)
      {
        leds[light_num[row][col] - 1] = color;
      }
      FastLED.show();
      get_speed(&delay_time);
      FastLED.delay(delay_time);
      if (RGB_mode_change_flag == 1)
      {
        RGB_mode_change_flag = 0;
        return;
      }
    }
  }
  // 熄灭所有RGB灯
  for (int i = 0; i < WS2812_NUMBER; i++) {
    leds[i] = CRGB::Black;
  }
  FastLED.show();
  FastLED.delay(delay_time * 10);
  if (RGB_mode_change_flag == 1)
  {
    RGB_mode_change_flag = 0;
    return;
  }
  for (int col = KEYBOARD_COLUMN - 1; col >= 0; col--) {
    CRGB color = CHSV(random8(), 255, 255);
    for (int row = KEYBOARD_ROW - 1; row >= 0; row--) {
      if (light_num[row][col] != ERROR_CODE)
      {
        leds[light_num[row][col] - 1] = color;
      }
      FastLED.show();
      get_speed(&delay_time);
      FastLED.delay(delay_time);
      if (RGB_mode_change_flag == 1)
      {
        RGB_mode_change_flag = 0;
        return;
      }
    }
  }
  // 熄灭所有RGB灯
  for (int i = 0; i < WS2812_NUMBER; i++) {
    leds[i] = CRGB::Black;
  }
  FastLED.show();
  FastLED.delay(delay_time * 10);
  if (RGB_mode_change_flag == 1)
  {
    RGB_mode_change_flag = 0;
    return;
  }
}

//旋转效果
void user_mode10()
{
  static uint8_t hue = 0;
  static uint8_t angle = 0;

  int center_row = KEYBOARD_ROW / 2;
  int center_col = KEYBOARD_COLUMN / 2;
  uint8_t delay_time = 0;
  get_speed(&delay_time);

  for (int row = 0; row < KEYBOARD_ROW; row++) {
    for (int col = 0; col < KEYBOARD_COLUMN; col++) {
      int index = light_num[row][col] - 1;

      int delta_row = row - center_row;
      int delta_col = col - center_col;
      float distance = sqrt(delta_row * delta_row + delta_col * delta_col);

      float angle_rad = atan2(delta_row, delta_col);
      float angle_offset = sin(angle + distance / 4) * PI / 6;
      angle_rad += angle_offset;

      int angle_deg = angle_rad / PI * 180;
      if (angle_deg < 0) angle_deg += 360;
      angle_deg /= 15;

      leds[index] = CHSV(hue + angle_deg * 10, 255, 255);
    }
  }

  hue += 5;
  angle += 0.1;
  FastLED.show();
  get_speed(&delay_time);
  FastLED.delay(delay_time);
}

//混色效果
void user_mode11()
{
  static uint8_t hue = 0;
  static uint8_t offset = 0;
  uint8_t delay_time = 0;
  get_speed(&delay_time);

  for (int row = 0; row < KEYBOARD_ROW; row++) {
    for (int col = 0; col < KEYBOARD_COLUMN; col++) {
      int index = light_num[row][col] - 1;
      int distance = abs(row - 2) + abs(col - 8);
      int angle = (offset + distance) % 24;
      leds[index] = CHSV(hue + angle * 10, 255, 255);
    }
  }

  hue += 2;
  offset++;

  if (offset >= 24) {
    offset = 0;
  }
  FastLED.show();
  get_speed(&delay_time);
  FastLED.delay(delay_time);
}


// Define the line properties
float linePhase = 0.0;
float lineSpeed = 0.03;
float lineAmplitude = 3.0;
float hueOffset = 0.0;
//波浪线效果
void user_mode12()
{
  uint8_t delay_time = 0;
  get_speed(&delay_time);
  // Update the line phase and hue offset
  linePhase += lineSpeed;
  hueOffset += 0.5;

  // Clear the LED array
  for (int i = 0; i < WS2812_NUMBER; i++) {
    leds[i] = CRGB::Black;
  }

  // Draw the zigzag line
  int y = KEYBOARD_ROW / 2;
  for (int x = 0; x < KEYBOARD_COLUMN; x++) {
    float hue = fmod(x * 10.0 + hueOffset, 256.0);
    if (x % 2 == 0) { // if even x coordinate
      y += int(sin(x / 2.0 + linePhase) * lineAmplitude);
    } else { // if odd x coordinate
      y += int(cos(x / 2.0 + linePhase) * lineAmplitude);
    }
    if (y < 0) y = 0;
    if (y >= KEYBOARD_ROW) y = KEYBOARD_ROW - 1;
    leds[light_num[y][x] - 1] = CHSV(hue, 255, 255);
  }

  // Show the LEDs
  FastLED.show();

  // Delay for a short time
  FastLED.delay(delay_time / 5);

}


//游戏模式
void user_gamemode()
{
  if (game_flag == 1)
  {
    game_flag = 0;
    for (int i = 0; i < WS2812_NUMBER; i++) {
      leds[i] = CRGB::Black;
    }
  }
  leds[esc_led] = CRGB::Blue;
  leds[w_led] = CRGB::Blue;
  leds[a_led] = CRGB::Blue;
  leds[s_led] = CRGB::Blue;
  leds[d_led] = CRGB::Blue;
  leds[up_led] = CRGB::Blue;
  leds[down_led] = CRGB::Blue;
  leds[left_led] = CRGB::Blue;
  leds[right_led] = CRGB::Blue;
  FastLED.show(); //刷新LED
}
